/***************************************************************************
 * Ballot.java                                                             *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *      
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * This is the definition for all the contents of a ballot,                 *
 * The Ballot class primarily handles navigation between pages, and         *
 * passing events to the currently selected  Page, which delegates the task *
 * to the Contest(See Contest.java for implementation details).             *
 *                                                                          *
 ****************************************************************************/


import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class Ballot{

    private Vector<Page> pages;
    private Vector<ReviewPage> reviewPages;
    private Vector<Contest> contests;
    private TreeMap<Integer,Integer> contestPageMapping;
    
    private Log log;
    
    private Integer currentPageNum;
    private Integer currentSummaryPageNum;
    
    private boolean isDishonest;
    private boolean reviewMode;
    private boolean hasUpdates;

    private boolean writeInMode;
    private String writeInText;
    private int writeInContest;
    private int writeInCand;

    private Integer numContestPages;

    
    public Ballot(Log l, Vector<Page> p, Vector<ReviewPage> rp, 
		  Vector<Contest> c , TreeMap<Integer,Integer> cpm){

	this.log                = l;
	this.pages              = p;
	this.reviewPages        = rp;
	this.contests           = c;
	this.contestPageMapping = cpm;
	
	this.setDishonest(false);
	this.reviewMode  = false;
	this.hasUpdates  = false;

	this.writeInMode = false;
	this.writeInText = "";
	this.writeInContest = 0;
	this.writeInCand    = 0;

	this.numContestPages = this.pages.size();
	this.currentPageNum = 0;
	this.currentSummaryPageNum = 0;


	if(this.pages.size() > 1)
	    pages.firstElement().disablePrev();

	if(this.reviewPages.size() > 1){
	    reviewPages.firstElement().disablePrev();
	    reviewPages.lastElement().setFinalPage();
	}
	
	this.pages.addAll( this.reviewPages);
	
	if( this.reviewPages.size() > 2)
	    pages.elementAt(pages.size() -2).setFinalNav();
    }

    
    public void reset(){
	this.reviewMode = false;
	for( Contest c : this.contests)
	    c.reset();
	
	this.navigateToPageSilent(0);
	
	for( Page p : this.reviewPages){
	    p.initDishonesty(false);
	    p.update();
	}
	
    }
    
    public boolean hasNextPage  () {return this.currentPageNum + 1 < this.getNumPages();}
    public boolean hasPrevPage  () {return this.currentPageNum - 1 >= 0;}    
    public boolean hasUpdates   () {return this.hasUpdates;}
    public boolean inReviewMode () {return this.reviewMode;}
    public boolean isFirstPage  () {return this.getCurrentPageNum() == 0;}
    public boolean atLastPage   () {return this.getCurrentPageNum() == this.pages.size()-1;}
    public boolean atTextPage   () {return this.getCurrentPage().hasText();}
    public boolean isWriteInMode() {return this.writeInMode;}
    
    public boolean isUpdatingContest(){
	return this.reviewMode && this.currentPageNum != currentSummaryPageNum;
    }
    
    public boolean atSummaryPage(){
	return inReviewMode() && this.currentPageNum < this.pages.size() - this.reviewPages.size() -1; 
    }
    
    public boolean atReviewPage(){
        return this.currentPageNum >= this.pages.size() - this.reviewPages.size() -1;
    }


    
    
    public void navigateToPage(int pageNum, String message){
	this.log.log("Navigate", message);
	navigateToPageSilent(pageNum);
    }
    
    
    public void navigateToPageSilent(int pageNum){
        this.hasUpdates = true;
        this.currentPageNum = pageNum;

        if( this.atReviewPage())
            this.reviewMode = true;
        if( this.atReviewPage() && this.isDishonest ){
            this.reviewPages.elementAt(0).setDishonest();
            this.reviewPages.elementAt(0).update();
        }
    }

    public void navigateToContest( int contestNum, String msg){
	try{
	    this.navigateToPage( this.contestPageMapping.get(contestNum), msg);}
	catch(NullPointerException e) {
	    System.err.printf("Ballot: Contest number %d does not exist\n", contestNum);}
	catch(ClassCastException e){ 
	    System.err.printf("Ballot.navigateToContest: Non Integer contestNum.\n");}
    }
    
    public void navigateToSummary(){
        navigateToPage( this.currentSummaryPageNum, "Return to Summary Screen");
    }
    
    public void navigateNext(){
	if(hasNextPage())
	    navigateToPage( getCurrentPageNum() + 1, "");
    }
    
    public void navigatePrev(){
	if(hasPrevPage())
	    navigateToPage( getCurrentPageNum() - 1, "");
    }
    
    public void reviewContest(int i){
        this.currentSummaryPageNum = this.getCurrentPageNum();
        this.navigateToContest(i, "Reviewing contest " + Integer.toString(i));
    }
    
    


    public Page getCurrentPage(){
	Page p = new Page(this.log);
	try{
	    p = this.pages.elementAt( this.currentPageNum );}
	catch(ArrayIndexOutOfBoundsException e){
	    System.err.printf("The ballot does not have page %d.\n", this.currentPageNum);
	    System.err.printf("Returning an empty page to keep PvPrime for crashing\n");
	}
	return p;
    }

    public ReviewPage getCurrentReviewPage(){
        ReviewPage p = null;
        try{
            p = this.reviewPages.elementAt( this.currentPageNum - (numContestPages ));}
        catch(ArrayIndexOutOfBoundsException e){
            System.err.printf("The ballot does not have review page %d.\n", this.currentSummaryPageNum);
            System.err.printf("Returning an empty page to keep PvPrime for crashing\n");
        }
        return p;
    }


    
    public Integer getCurrentPageNum(){ return this.currentPageNum;}
    public Integer getNumPages()      { return this.pages.size();}
    public Integer getNumContests()   { return this.contestPageMapping.size();}
    
    public String getWriteInText(){return this.writeInText;}
    public void   setWriteInText(String s){ 
	this.hasUpdates = true;
	this.writeInText = s;
    }

    public void setCurrentWriteIn(int contestNum, int candNum){
	if( contestNum < this.contests.size()){
	    this.writeInContest = contestNum;
	    this.writeInCand    = candNum;
	    this.setWriteInMode(true);
	}
    }


    public void acceptWriteIn(){
	Contest   cont = this.contests.elementAt(writeInContest);
	Candidate cand = cont.getCandidate(writeInCand);
	
	cand.setTitle(this.writeInText);
	cont.select( cand );
	this.setWriteInMode(false);
	this.hasUpdates = true;
    }


    public void updateContest(int contestNum, int option){
	this.hasUpdates = true;
	this.contests.get(contestNum).pressed(option);
    }
    

    public void setDishonest(boolean b){
	if( !this.reviewPages.isEmpty())
	    this.reviewPages.firstElement().initDishonesty( b );
	this.isDishonest = b; 
    }

    
    public void setWriteInMode(boolean b){
	this.hasUpdates = true;
	this.writeInMode = b;
    }


    public void updatesComplete(){
	this.hasUpdates = false;
    }

}
