/***************************************************************************
 * Contest.java                                                            *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * Define the elements of a contest within a given ballot page.             *
 * This handles storage of candidates, informational, candidate selection,  *
 * swapping selections for simulating a dishonest summary screen.           *
 ****************************************************************************/


import java.util.*;

public class Contest {

    protected Log log;
    
    private Integer contestNum;
    private Integer maxSelections;
    private Integer numSelections;
    
    protected String sectionText;
    protected String raceText;
    protected String instructionText;
    
    protected boolean writeInEnabled;
    protected boolean twoPanelView;
    protected boolean hideSectionText;
    
    protected LinkedList<Candidate> candidates;
    
    
    public Contest(){}
    
    public Contest(Log l, Integer maxSel, String sText, String rText,
		   int contestNum, boolean writeIn, boolean twoColView, 
		   boolean hideSection, LinkedList<Candidate> cands){
	this.log = l;

	this.writeInEnabled  = writeIn;
	this.twoPanelView    = twoColView;
	this.hideSectionText = hideSection;

	this.sectionText     = sText;
	this.raceText        = rText;
	this.instructionText = "";
	
	this.contestNum    = contestNum;
	this.maxSelections = maxSel;
	this.numSelections = 0;
	this.candidates    = cands;

	if( writeInEnabled){
	    int size = cands.size();
	    for(int i = size ; i < size + maxSelections ; i++)
		this.candidates.add( new Candidate("","","",i,writeInEnabled) );
	}

    }
    
    public void reset(){
	this.numSelections = 0;

	for( Candidate c : candidates){
	    if(c.isWriteIn())
		c.setTitle("");
	    c.setSelected(false);
	}
    }
    
    public void setSectionText    (String s){ this.sectionText = s;}
    public void setRaceText       (String s){ this.raceText = s;}
    public void setInstructionText(String s){ this.instructionText = s;}


    public LinkedList<Candidate> getCandidates(){
	return this.candidates;
    }
  
    public Integer getContestNum()      { return this.contestNum;}
    public Integer getMaxSelections()   { return this.maxSelections;}
    public boolean atMaxSelections()    { return maxSelections == numSelections;}
    public String  getSectionText()     { return this.sectionText;}
    public String  getRaceText()        { return this.raceText;}
    public String  getInstructionText() { return this.instructionText;}
    public boolean isSectionTextHidden(){ return this.hideSectionText;}
    public boolean isWriteInEnabled()   { return this.writeInEnabled;}
    public boolean isTwoPanelView()     { return this.twoPanelView;}
    public boolean isSelected(int c)    { return this.getCandidate(c).isSelected();}


    public Candidate getCandidate(int c){
	Candidate cand = null;
	try{ cand = this.candidates.get(c); }
	catch(IndexOutOfBoundsException i){
	    System.err.println("Contest.getCandidate: out of bounds");
	    System.exit(1);
	}
	return cand;
    }

    public void pressed(int c){
	Candidate cand = this.getCandidate(c);

	if( (numSelections < maxSelections) && !cand.isSelected() )
	    this.select(cand);
	else if(cand.isSelected())
	    this.deselect(cand);
	/* This occurs when someone tries to overvote
	 * The log processer currently detects this implictly. This included for 
	 * documentation, although it may be usful to log this explicitly.*/ 
	//else if( (numSelections == maxSelections) && !cand.isSelected() )
	    //this.log.log("ISelect","");

    }
    
    public void select(Candidate cand){
	this.log.log("Select","true");
	cand.setSelected(true);
	numSelections++;
    }

    public void deselect(Candidate cand){
	this.log.log("Select","false");
	cand.setSelected(false);
	numSelections--;
    }
    
    public void setDishonestCandidate(){
	this.log.log("Dishonest", this.getRaceText());
	if( candidates.size() > 0 && numSelections == 0)
	    candidates.getFirst().setSelected(true);
	else if(candidates.size() > 1 && candidates.getFirst().isSelected()){
	    candidates.getFirst().setSelected(false);
	    candidates.get(1).setSelected(true);
	}
	else if(candidates.size() > 0 && !candidates.getFirst().isSelected()){
	    for(Candidate c : candidates)
		c.setSelected(false);
	    candidates.getFirst().setSelected(true);	    
	}
    }
}
