/***************************************************************************
 * Log.java                                                                *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/
/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * The Log class handles printing all information to the event log along    *
 * with most of the xml formatting and timestamps.                          *
 *                                                                          *
 * Log contains three private helper classes:                               *
 *  - EventLog: The information for 1 event. This is essentially a list     *
 *              of EventLines.                                              *
 *  - EventLine: This contains one piece of information about an event,     *
 *               such as relative coordinates, was a selection made, etc,   *
 *               along with a timestamp.                                    *
 *****************************************************************************/



import java.util.*;
import java.io.*;
import java.awt.event.*;

public class Log{

    private final int ONE_BILLION = 1000000000;
    
    private long relTime;
    private PrintStream log;
    private StringBuffer sb;
    
    private EventLog event;
    
    public Log( String text){
	this.relTime = 0;
	this.sb      = new StringBuffer();
	this.event   = new EventLog();  
	
	
	try{
	    Date d = new Date();
	    String s = d.toString().replaceAll(" ", "-");
	    s = s.replaceAll(":", "");
	    s = s.replace("Sat-", "");
	    s = s.replace("-CDT-2009", "");
	    
	    this.log  = new PrintStream( "logs/" +text +"-" + s + ".xml");
	    
	    this.log.println("<?xml version=\"1.0\"?>\n<Log>");
	}
	catch(FileNotFoundException fe){
	    System.err.println("Failed to open file");
	    System.exit(1);
	}
    }
    
    
    
    
    private String timeFormat(){
	return String.format("%4.3f", (this.getRelTime() / ONE_BILLION) );
    }
    
    private Long getRelTime(){
	return System.nanoTime()- relTime;
    }
    
    public void setRelativeTime(){
	this.relTime = (System.nanoTime());
    }
    
    public synchronized void startEvent(){
	if ( !this.event.isEmpty())
	    this.endEvent();
    }
    
    public synchronized void endEvent(){
	this.log.println(this.event.toXmlString());
	this.event.clear();
    }

    public synchronized void log(String n, String o){
	this.event.addEvent(n,o, this.getRelTime());
    }
    
    public synchronized void print(String s){
	this.log.print(s);
    }
    
    public synchronized void println(String s){
	this.log.println(s);
    }
    
    public void close(){
	this.log.printf("</Voter>\n</Log>");
    }
    
    
    
    private class EventLine{
	private String name;
	private String option;
	private Long   time;
	
	public EventLine(String n, String o, Long t){
	    name = n;
	    option = o;
	    time = t;
	}
	
	public String toXmlString(){
	    StringBuffer sb = new StringBuffer();
	    
	    sb.append("<").append(name).append(" time=\"");
	    sb.append(time.toString() ).append("\">");
	    
	    sb.append(option);
	    
	    sb.append("</").append(name).append(">");
	    return sb.toString();
	}
    }
    
    
    
    
    private class EventLog{
	LinkedList<EventLine> subEvents;
	
	public EventLog(){
	    this.subEvents = new LinkedList<EventLine>();
	}
	
	public void addEvent(String n, String o, Long t){
	    this.subEvents.add( new EventLine(n,o,t) );
	}
	
	public void clear()   { this.subEvents.clear();}
	public boolean isEmpty() { return this.subEvents.isEmpty();}
	
	public String toXmlString(){
	    StringBuffer sb = new StringBuffer();
	    
	    sb.append("<Event>\n");
	    for(EventLine e : subEvents)
		sb.append( e.toXmlString() ).append("\n");
	    sb.append("</Event>");
	    
	    return sb.toString();
	}
	
    }
}
