/*****************************************************************************
 * Model.java                                                                *
 *****************************************************************************/

/******************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                                 *
 *                                                                            *
 * Permission is hereby granted, free of charge, to any person obtaining a    *
 * copy of this software and associated documentation files (the              *
 * "Software"), to deal in the Software without restriction, including        *
 * without limitation the rights to use, copy, modify, merge, publish,        *
 * distribute, distribute with modifications, sublicense, and/or sell         *
 * copies of the Software, and to permit persons to whom the Software is      *
 * furnished to do so, subject to the following conditions:                   *
 *                                                                            *
 * The above copyright notice and this permission notice shall be included    *
 * in all copies or substantial portions of the Software.                     *
 *                                                                            *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                 *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.     *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,     *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR      *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR      *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                 *
 *                                                                            *
 * Except as contained in this notice, the name(s) of the above copyright     *
 * holders shall not be used in advertising or otherwise to promote the       *
 * sale, use or other dealings in this Software without prior written         *
 * authorization.                                                             *
 ******************************************************************************/

/*******************************************************************************
 * Paul Cotton                                                                 *
 * The University of Iowa                                                      *
 * Department of Computer Science                                              *
 * April 2009                                                                  *
 *                                                                             *
 * The model handles selection of test modes(Control, Dishonest, Miscalibrated *
 * Delayed and Compressed.), and the ballot definition needed for that mode    *
 *******************************************************************************/


import java.awt.event.*;
import java.util.*;

public class Model{
    
    private enum TestMode {HONEST, CALIBRATION_P, CALIBRATION_N,  DELAY, DISHONEST, BANNER_BLIND}
    
    private Integer[] delays  = {100, 250};
    private Double[]  offsets = {0.15,0.3};
    
    private Log    log;
    private DataEntry de;

    private Ballot ballot;
    private Ballot standardBallot;
    private Ballot compressedBallot;
    
    private TestMode mode;
    
    private Integer numVoters;
    
    private Random random;
    private Double  calibration;
    private Integer delay;
    
  
    public Model(String uncompressed, String compressed, Log l){
	XmlBallotParser b = new XmlBallotParser( l );
	
	this.log = l;
	
	this.mode = TestMode.HONEST;
	
	this.standardBallot     = b.getBallot(uncompressed);
	this.compressedBallot   = b.getBallot(compressed);
	this.ballot = this.standardBallot;
	this.numVoters = 1;
	
	this.random = new Random();
	this.calibration = 0.0;
	this.delay       = 0;
    }
    
    public  void initNewSession(){
	
	this.ballot      = this.selectBallot();
	this.calibration = this.selectCalibration();
	this.setDelay    (this.selectDelay());
	
	this.ballot.setDishonest( this.mode == TestMode.DISHONEST);
    }
    
    public void  demographicsEntered(){
	this.ballot.reset();
	this.numVoters++;
    }
    
    private Ballot selectBallot(){
	Ballot b = this.standardBallot;
	if( this.mode == TestMode.BANNER_BLIND)
	    b = this.compressedBallot;


	return b;
    }
    
    private Double selectCalibration(){
	Double cal = this.offsets[this.random.nextInt(offsets.length)];
	if( this.mode == TestMode.CALIBRATION_N)
	    cal = -1 * cal;
	return cal;
  }
    
    
    private Integer selectDelay(){
	Integer del = 0;
	if (this.mode == TestMode.DELAY)
	    del = this.delays[ this.random.nextInt(delays.length) ];
	return del;
    }
    
    public void selectDemoMode(KeyEvent e){
	int key = e.getKeyCode() % 96;
	
	switch( key ){
	case 0 : this.mode = TestMode.HONEST;break;
	case 1 : this.mode = TestMode.CALIBRATION_P;break;
	case 2 : this.mode = TestMode.CALIBRATION_N;break;
	case 3 : this.mode = TestMode.DELAY;break;
	case 4 : this.mode = TestMode.DISHONEST;break;
	case 5 : this.mode = TestMode.BANNER_BLIND;break;
	default: this.mode = TestMode.HONEST;break;
	}
    }
    
    public void selectMode(){
	int sel = this.random.nextInt(5);
	

	switch( sel){
	case 0 : this.mode = TestMode.HONEST; break;
	case 1 : this.mode = TestMode.CALIBRATION_P;break;
	case 2 : this.mode = TestMode.CALIBRATION_N;break;
	case 3 : this.mode = TestMode.DELAY;break;
	case 4 : this.mode = TestMode.DISHONEST;break;
	case 5 : this.mode = TestMode.BANNER_BLIND;break;
	default: this.mode = TestMode.HONEST;break;
	}
    }
    
    
    public String getMode(){
	return this.mode.toString();
    }

    public Integer getNumVoters(){
	return this.numVoters;
    }
    
    public String getModeOptions(){
	String m = "";
	
	if(this.mode == TestMode.DELAY)
	    m = this.delay.toString() ;
	else if(this.mode == TestMode.CALIBRATION_P || this.mode == TestMode.CALIBRATION_N)
	    m = Integer.toString(((int)((double)this.calibration * FakeButton.HEIGHT)));
	return m;
    }

    
    private synchronized void setDelay(int d){
	this.delay = d;
    }
    
    
    //public              boolean isDishonest()  {return this.mode == TestMode.DISHONEST;}
    public boolean isInsensitive(){return this.mode == TestMode.DELAY;}
    public boolean isMiscalibrated(){
	return this.mode == TestMode.CALIBRATION_P || this.mode == TestMode.CALIBRATION_N;
    }

    public MouseEvent miscalibrate(MouseEvent m){
	m.translatePoint(0, (int)(this.calibration * FakeButton.HEIGHT));
	return m;
    }

    public int    getDelayTime() {return this.delay;}
    public Ballot getBallot()    {return this.ballot;}

}
